/* Connects to server and downloads a file using byte ranges. */
/* This program is intended to be used for testing. */

//#define USEWIN32*/
#define USEAROS

#ifdef USEWIN32
#include <winsock2.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
/*#include <mem.h>*/

#if defined (USEUNIX) || defined (USEBEOS) || defined (USEAROS)
#include <unistd.h>
#include <sys/time.h>
#endif /* #if defined (USEUNIX) || defined (USEBEOS) || defined (USEAROS) */


#ifdef USEAROS
/* Network includes */
#include <proto/socket.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <proto/exec.h>          /* OpenLibrary() and CloseLibrary() */
#include <exec/libraries.h>      /* struct Library */
struct Library *SocketBase; /* Socket library handle. Needs to be global. */
#endif /* USEAROS */

#ifndef SHUT_WR
	#define SHUT_WR 1
#endif

int ConnectToServer(char *ip, int port);
int CloseConnection(int sd);

#ifdef USEWIN32
int InitWinsock(void) {
	WORD wVersionRequested = MAKEWORD(1,1);
   WSADATA wsaData;

	if (WSAStartup(wVersionRequested, &wsaData) != 0) {
      puts("ERROR - Could not Initialize Winsock.");
      return -1;
   }
   return 0;
}
#endif

/* Closes connection sd. Returns 0 on success or -1 on failure. */
int CloseConnection(int sd) {
	int nNewBytes;
   char buf[256];

   if( shutdown(sd, SHUT_WR) < 0 ) {
   	/* shutdown failed. */
      /* Ignore this as we must free the resources associated with sd. */
   }

   while( 1 ) {
   	nNewBytes = recv(sd, buf, sizeof(buf), 0);
      if( nNewBytes < 0 ) { break; } /* Connection reset by peer. */
      else if( nNewBytes != 0 ) {} /* Unexpected bytes during shutdown. */
      else { break; }
   }

   #ifdef USEWIN32
   if(closesocket(sd) < 0) { return -1; } /* Closing connection failed */
   #endif
   #ifdef USEOS2
   if(soclose(sd) < 0) { return -1; } /* Closing connection failed */
   #endif
   #if defined (USEUNIX) || defined (USEAROS)
   if(close(sd) < 0) { return -1; } /* Closing connection failed */
   #endif

   return 0;
}

int InputString(char *string, int len) {
   int i;

   fflush(stdin);
	if( fgets(string, len, stdin) == NULL ) { return -1; }
   else {
   	for(i=0;i<len;i++) {
      	if ( string[i] == '\n' ) {
            string[i] = '\0';
            break;
      	}
      }
   }
   return 0;
}


int main(void) {
   char ip[16], Method[16], ByteRange[32], FileName[64], ServerPort[6];
   int sd;
   char buf[512];
   char *ptr;
   int retval, nBytesToSend, i;

   #ifdef USEWIN32
   if( InitWinsock() != 0 ) {
   #endif
   #ifdef USEOS2
   /* Init socket stuff (seems like it's not necessary but used anyway). */
   if( sock_init() == -1 ) {
   #endif
   #ifdef USEAROS
   SocketBase = (struct Library *)OpenLibrary("bsdsocket.library", 3);
   if( SocketBase == NULL ) {
   #endif
   	return 0;
   }

   strcpy(ip, "127.0.0.1");

   printf("Enter server IP address or q to quit program.\n: ");
   if( InputString(ip, sizeof(ip)) == -1 ) { return 0; }

   if( strcmp(ip, "q") == 0 ) { return 0; }

   printf("Enter server port.\n: ");
   if( InputString(ServerPort, sizeof(ServerPort)) == -1 ) { return 0; }

   printf("Enter HTTP Method.\n: ");
   if( InputString(Method, sizeof(Method)) == -1 ) { return 0; }

   printf("Enter byte range field or x to skip.\n: ");
   if( InputString(ByteRange, sizeof(ByteRange)) == -1 ) { return 0; }

   printf("Enter file name (/filename).\n: ");
   if( InputString(FileName, sizeof(FileName)) == -1 ) { return 0; }

   if( strcmp(ByteRange, "x") == 0 ) {
   	sprintf(buf, "%s %s HTTP/1.0\r\n\r\n", Method, FileName);
   }
   else {
   	sprintf(buf, "%s %s HTTP/1.0\r\nRange: bytes=%s\r\n\r\n", Method, FileName, ByteRange);
   }
         
   if( (sd = ConnectToServer(ip, atoi(ServerPort))) != -1 ) {
   	puts("Connected to server.");
      printf("Sending:\n%s", buf);
      nBytesToSend = strlen(buf);
      while( nBytesToSend > 0 ) {
      	retval = send(sd, buf, nBytesToSend, 0);
         if( retval == 0 ) {
         	puts("\nConnection closed by server.");
         	return 0;
         }
         else if( retval < 0 ) {
         	puts("Error on socket.");
            return 0;
         }
         else {
         	printf("%d B sent to server.\n", retval);
         	nBytesToSend -= retval;
            if( nBytesToSend == 0 ) {
            	puts("\nBuffer sent.");
            }
         }
      }

      do {
      	memset(buf, 0, sizeof(buf));
         retval = recv(sd, buf, sizeof(buf) - 1, 0);
         if( retval == 0 ) {
         	puts("\nConnection closed by server.");
            return 0;
         }
         else if( retval < 0 ) {
         	puts("Error on socket.");
            return 0;
         }
         else {
         	printf("%d bytes recv from server.\n", retval);
            for(i=0;i<retval;i++) {
            	printf("%c", buf[i]);
            }
         }
      } while( retval > 0 ) ;

      CloseConnection(sd);
         /*
         ptr = strstr(buf, "\r\n\r\n");
         if( ptr != NULL ) {
         	*ptr = '\0';
            puts(buf);
         }
         */
   }
   
   #ifdef USEAROS
   CloseLibrary( SocketBase ); /* Close libraries. */
   #endif/* USEAROS */
   #ifdef USEWIN32
   WSACleanup();
   #endif

   return 0;
}


int ConnectToServer(char *ip, int port) {
   struct sockaddr_in serv;
   int sd; /* socket descriptor. */

   /* Creating socket */
   sd = socket(AF_INET, SOCK_STREAM, 0);
   if( sd == -1 ) {
   	puts("Creating socket failed.");
   	return -1;
   }

   /* fill-in address information */
   serv.sin_family = AF_INET;
   serv.sin_port = htons((short)port);
   serv.sin_addr.s_addr = inet_addr(ip);

   /* connect to the server */
   #ifdef USEAROS
   if( connect(sd, (struct sockaddr *)&serv, sizeof(struct sockaddr)) != 0 ) {
   #else
   if( connect(sd, (LPSOCKADDR)&serv, sizeof(struct sockaddr)) != 0 ) {
   #endif
   	puts("Connect failed.");
      return -1;
   }

   return sd;
}


