/* This program is intended to be used for testing. */

/* Compile for windows WIN32 plattform. */
#if defined (__WIN32__) && defined (_Windows)
  #define USEWIN32
#endif

/* Compile for *NIX plattform. */
/*#define USEUNIX*/

/* Compile for BeOS. */
/*#define USEBEOS */

/* Compile for OS/2 Warp 3 Connect using Open Watcom 1.3). */
/* #define USEOS2 */

/* Compile for Windows for Workgroups 3.11. */

#if !defined (__WIN32__) && defined (_Windows)
  #define USEWIN16 */
#endif

/* Compile for The Amiga Research Operating System (AROS). */
#ifdef __AROS__
  #define USEAROS
#endif


/* ************************************************************************** */

#ifdef USEWIN32
#include <winsock2.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
/*#include <mem.h>*/

#if defined (USEUNIX) || defined (USEBEOS) || defined (USEAROS)
#include <unistd.h>
#include <sys/time.h>
#endif /* #if defined (USEUNIX) || defined (USEBEOS) || defined (USEAROS) */


#ifdef USEAROS
/* Network includes */
#include <proto/socket.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/errno.h>
#include <net/if.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <proto/exec.h>          /* OpenLibrary() and CloseLibrary() */
#include <exec/libraries.h>      /* struct Library */
struct Library *SocketBase; /* Socket library handle. Needs to be global. */
#endif /* USEAROS */

#ifndef SHUT_WR
	#define SHUT_WR 1
#endif

int ConnectToServer(char *ip, int port);
int CloseConnection(int sd);

#ifdef USEWIN32
int InitWinsock(void) {
	WORD wVersionRequested = MAKEWORD(1,1);
   WSADATA wsaData;

	if (WSAStartup(wVersionRequested, &wsaData) != 0) {
      puts("ERROR - Could not Initialize Winsock.");
      return -1;
   }
   return 0;
}
#endif

/* Closes connection sd. Returns 0 on success or -1 on failure. */
int CloseConnection(int sd) {
	int nNewBytes;
   char buf[256];

   if( shutdown(sd, SHUT_WR) < 0 ) {
   	/* shutdown failed. */
      /* Ignore this as we must free the resources associated with sd. */
   }

   while( 1 ) {
   	nNewBytes = recv(sd, buf, sizeof(buf), 0);
      if( nNewBytes < 0 ) { break; } /* Connection reset by peer. */
      else if( nNewBytes != 0 ) {} /* Unexpected bytes during shutdown. */
      else { break; }
   }

   #ifdef USEWIN32
   if(closesocket(sd) < 0) { return -1; } /* Closing connection failed */
   #endif
   #ifdef USEOS2
   if(soclose(sd) < 0) { return -1; } /* Closing connection failed */
   #endif
   #if defined (USEUNIX) || defined (USEAROS)
   if(close(sd) < 0) { return -1; } /* Closing connection failed */
   #endif

   return 0;
}

int InputString(char *string, int len) {
   int i;

   fflush(stdin);
	if( fgets(string, len, stdin) == NULL ) { return -1; }
   else {
   	for(i=0;i<len;i++) {
      	if ( string[i] == '\n' ) {
            string[i] = '\0';
            break;
      	}
      }
   }
   return 0;
}

int main(void) {
   char ip[16], ServerPort[6];
   int sd, dsd; /* Command connection and data connection. */
   char FTPCmnd[512], buf[512];
   char *File = "This is a fake file.";
   char *ptr;
   int retval, nBytes, i;
	#ifdef USEAROS
   char nonBlock = 1;
   #endif
   #if defined (USEWIN32) || defined (USEWIN16) || defined (USEUNIX) || defined (USEBEOS) || defined (USEOS2)
   unsigned long nonBlock = 1;
   #endif


   #ifdef USEWIN32
   if( InitWinsock() != 0 ) {
   #endif
   #ifdef USEOS2
   /* Init socket stuff (seems like it's not necessary but used anyway). */
   if( sock_init() == -1 ) {
   #endif
   #ifdef USEAROS
   SocketBase = (struct Library *)OpenLibrary("bsdsocket.library", 3);
   if( SocketBase == NULL ) {
   #endif
   	return 0;
   }

   strcpy(ip, "127.0.0.1");

   printf("Enter server IP address or q to quit program.\n: ");
   if( InputString(ip, sizeof(ip)) == -1 ) { return 0; }

   if( strcmp(ip, "q") == 0 ) { return 0; }

   printf("Enter server port.\n: ");
   if( InputString(ServerPort, sizeof(ServerPort)) == -1 ) { return 0; }

   if( (sd = ConnectToServer(ip, atoi(ServerPort))) != -1 ) {
   	puts("Connected to server.");
   }
   else {
   	puts("Not connected.");
      return 0;
   }

   printf("Enter QUIT on a single line to close connection and exit program.\n: ");
   printf("Enter FTP command and arguments.\n");


   memset(FTPCmnd, 0, sizeof(FTPCmnd));

   do {
      printf(">");
      /* Going into non-blocking. */
		#if defined (USEWIN32) || defined (USEWIN16)
   	if( ioctlsocket(sd, FIONBIO, &nonBlock) != 0) {
   		fputs("\nSetting socket to non-blocking mode failed.\n", stderr);
      	return -1;
   	}
   	#endif
      #ifdef USEBEOS
      if( setsockopt(sd, SOL_SOCKET, SO_NONBLOCK, &nonBlock, sizeof(nonBlock)) != 0) {
   	   fputs("\nSetting socket to non-blocking mode failed.\n", stderr);
         return -1;
      }
      #endif /* #ifdef USEBEOS */
      #ifdef USEAROS
      if( IoctlSocket(sd, FIONBIO, &nonBlock) != 0) {
     	   fputs("\nSetting socket to non-blocking mode failed.\n", stderr);
     	   return -1;
      }
      #endif /* #ifdef USEAROS */

      #ifdef USEUNIX
      if( ioctl(sd, FIONBIO, &nonBlock) != 0) {
        	fputs("\nSetting socket to non-blocking mode failed.\n", stderr);
        	return -1;
      }
      #endif /* USEUNIX */
      #ifdef USEOS2
      if( ioctl(sd, FIONBIO, (char *)&nonBlock, sizeof(nonBlock)) != 0) {
      	fputs("\nSetting socket to non-blocking mode failed.\n", stderr);
         return -1;
      }
      #endif /* USEOS2 */

     	i = 0;
      nBytes = 0;
      memset(buf, 0, sizeof(buf));
      do {
         retval = recv(sd, buf+nBytes, sizeof(buf) - 1 - nBytes, 0);
         if( retval == 0 ) {
         	puts("\nConnection closed by server.");
            return 0;
         }
         else if( retval < 0 ) {
   			/* Error - could be a EWOULDBLOCK. */
				#if defined (USEWIN32) || defined (USEWIN16)
      		if( WSAEWOULDBLOCK != WSAGetLastError() ) {
      		#endif
      		#if defined (USEUNIX) || defined (USEBEOS)
				if( errno != EWOULDBLOCK ) {
      		#endif
      		#ifdef USEAROS
      		if( Errno() != EWOULDBLOCK ) {
      		#endif
      		#ifdef USEOS2
      		if( sock_errno() != SOCEWOULDBLOCK ) {
      		#endif
               puts("\nError on socket.");
         		break; /* Something else was wrong. */
            }
            else {
            }
         }
         else {
            for(i=0;i<retval;i++) {
            	printf("%c", buf[i+nBytes]);
               if( buf[i+nBytes] == '\n' ) {
               	i = -1;
						break;
               }
            }
            nBytes += retval;
         }
      } while( i != -1 ) ;

      if( strncmp(FTPCmnd, "PASV", 4) == 0 && strncmp(buf, "227 ", 4) == 0) {
			printf("Creating data connection.\nEnter server IP address.\n: ");
   		if( InputString(ip, sizeof(ip)) == -1 ) { return 0; }

   		printf("Enter server port.\n: ");
   		if( InputString(ServerPort, sizeof(ServerPort)) == -1 ) { return 0; }

   		if( (dsd = ConnectToServer(ip, atoi(ServerPort))) != -1 ) {
   			puts("Connected to server.");
   		}
   		else {
   			puts("Not connected.");
   		}
      }

      /* Going into blocking. */
      nonBlock = 0;
		#if defined (USEWIN32) || defined (USEWIN16)
   	if( ioctlsocket(sd, FIONBIO, &nonBlock) != 0) {
   		fputs("\nSetting socket to blocking mode failed.\n", stderr);
      	break;
   	}
   	#endif
      #ifdef USEBEOS
      if( setsockopt(sd, SOL_SOCKET, SO_NONBLOCK, &nonBlock, sizeof(nonBlock)) != 0) {
   	   fputs("\nSetting socket to blocking mode failed.\n", stderr);
         break;
      }
      #endif /* #ifdef USEBEOS */
      #ifdef USEAROS
      if( IoctlSocket(sd, FIONBIO, &nonBlock) != 0) {
     	   fputs("\nSetting socket to blocking mode failed.\n", stderr);
     	   break;
      }
      #endif /* #ifdef USEAROS */

      #ifdef USEUNIX
      if( ioctl(sd, FIONBIO, &nonBlock) != 0) {
        	fputs("\nSetting socket to blocking mode failed.\n", stderr);
        	break;
      }
      #endif /* USEUNIX */
      #ifdef USEOS2
      if( ioctl(sd, FIONBIO, (char *)&nonBlock, sizeof(nonBlock)) != 0) {
      	fputs("\nSetting socket to blocking mode failed.\n", stderr);
         break;
      }
      #endif /* USEOS2 */


   	printf("<");
      memset(FTPCmnd, 0, sizeof(FTPCmnd));
   	if( InputString(FTPCmnd, sizeof(buf)-10) == -1 ) { return 0; }
      strcat(FTPCmnd, "\r\n");
      nBytes = strlen(FTPCmnd);

      if( strncmp(FTPCmnd, "PORT ", 5) == 0 ) {

      }

      while( nBytes > 0 ) {
      	retval = send(sd, FTPCmnd, nBytes, 0);
         if( retval == 0 ) {
         	puts("\nConnection closed by server.");
         	return 0;
         }
         else if( retval < 0 ) {
         	puts("\nError on socket.");
            return 0;
         }
         else {
         	nBytes -= retval;
            if( nBytes == 0 ) { break; }
         }
      }
   } while( strcmp(buf, "QUIT") != 0 );

   CloseConnection(sd);
   #ifdef USEAROS
   CloseLibrary( SocketBase ); /* Close libraries. */
   #endif/* USEAROS */
   #ifdef USEWIN32
   WSACleanup();
   #endif

   return 0;
}


int ConnectToServer(char *ip, int port) {
   struct sockaddr_in serv;
   int sd; /* socket descriptor. */

   /* Creating socket */
   sd = socket(AF_INET, SOCK_STREAM, 0);
   if( sd == -1 ) {
   	puts("Creating socket failed.");
   	return -1;
   }

   /* fill-in address information */
   serv.sin_family = AF_INET;
   serv.sin_port = htons((short)port);
   serv.sin_addr.s_addr = inet_addr(ip);

   /* connect to the server */
   #ifdef USEAROS
   if( connect(sd, (struct sockaddr *)&serv, sizeof(struct sockaddr)) != 0 ) {
   #else
   if( connect(sd, (LPSOCKADDR)&serv, sizeof(struct sockaddr)) != 0 ) {
   #endif
   	puts("Connect failed.");
      return -1;
   }

   return sd;
}


